/*-----------------------------------------------------------
 * Function GetTextDimensions
 *
 * C-API wrapper for the GetTextExtentPoint32 WinAPI function,
 * which may replace GraQueryTextBox in some of its most frequent use cases.
 *
 * This is no plug-in replacement for GraQueryTextBox though,
 * since parameters and return types are incompatible.
 *
 * Link dependency: gdi32.lib
 *                  That file is available on the web, for example
 *                  as part of the Masm32 package from
 *                  http://www.masm32.com/masmdl.htm
 *
 * Usage
 * from Xbase++:    _GetTextDimensions( <hDC>, <cText>, <@nWidth>, <@nHeight> ) -> LOGICAL
 *
 * Parameters:      hDC     [in]  Device context handle
 *                  cText   [in]  String
 *                  nWidth  [out] Width of string in device units. Must be passed by reference.
 *                  nHeight [out] Height of string in device units. Must be passed by reference.
 *
 * Return value:    Success = TRUE, failure = FALSE (parameter error or otherwise)
 *
 * July 29, 2005
 *
 * Author: Gnter Beyes, XSYS Datenkommunikation GmbH
 *
 * --------------------------------------------------------*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <xppdef.h>
#include <xpppar.h>
#include <xppcon.h>
#include <error.ch>

XPPRET XPPENTRY GETTEXTDIMENSIONS(  XppParamList pList )
{
  ContainerHandle chDC = NULLCONTAINER ;
  ContainerHandle chText = NULLCONTAINER ;
  ContainerHandle chX = NULLCONTAINER ;
  ContainerHandle chY = NULLCONTAINER ;
  LONG hdc ;
  char * text ;
  int textsize ;
  SIZE dims = {0} ;
  BOOL ok = FALSE ;
  BOOL locked2 = FALSE ;
  BOOL byRef3 = FALSE ;
  BOOL byRef4 = FALSE ;

  if ( ! ( _partype( pList, 1 ) & XPP_NUMERIC) ) goto _quit ;
  if ( ! ( _partype( pList, 2 ) & XPP_CHARACTER) ) goto _quit ;
  if ( ! ( _partype( pList, 3 ) & XPP_NUMERIC) ) goto _quit ;
  if ( ! ( _partype( pList, 4 ) & XPP_NUMERIC) ) goto _quit ;

  if ( (chDC   = _conParam( pList, 1, NULL ) ) == NULLCONTAINER ) goto _quit ;
  if ( (chText = _conParam( pList, 2, NULL ) ) == NULLCONTAINER ) goto _quit ;
  if ( (chX    = _conParam( pList, 3, &byRef3 ) ) == NULLCONTAINER ) goto _quit ;
  if ( (chY    = _conParam( pList, 4, &byRef4 ) ) == NULLCONTAINER ) goto _quit ;

  // third and fourth parameters must be passed by reference
  if ( ( ! byRef3 ) || ( ! byRef4 ) ) goto _quit ;

  if ( _conGetNL( chDC, &hdc ) != XPP_ERR_NONE ) goto _quit ;

  if ( _conRLockC( chText, &text, &textsize ) != XPP_ERR_NONE ) goto _quit ;

  locked2 = TRUE ;

  if ( GetTextExtentPoint32( (HDC)hdc, text, textsize, &dims ) )
  {
     _conPutNL( chX, dims.cx ) ;
     _conPutNL( chY, dims.cy ) ;
     ok = TRUE ;
  }

  _quit:

  _retl( pList, ok );

  // unlock container
  if ( locked2 ) _conUnlockC( chText ) ;

  // release containers
  if (chDC != NULLCONTAINER) _conRelease( chDC ) ;
  if (chText != NULLCONTAINER) _conRelease( chText ) ;
  if ((chX != NULLCONTAINER) && (! byRef3)) _conRelease( chX ) ;
  if ((chY != NULLCONTAINER) && (! byRef4)) _conRelease( chY ) ;

  return ;

}
